==> fred-hermes-0.4.0.tar.gz: fred-hermes-0.4.0/hermes/tests/test_client.py <==
"""Unittests for MessengerClient."""
from datetime import datetime
from unittest import TestCase
from unittest.mock import call, sentinel
from uuid import UUID

import grpc
from fred_api.messenger.common_types_pb2 import Uuid
from fred_api.messenger.email_pb2 import Email as MessengerEmail
from fred_api.messenger.service_email_grpc_pb2 import (GetEmailReply, GetEmailRequest, ListEmailReply, ListEmailRequest,
                                                       SendEmailReply, SendEmailRequest)
from frgal.utils import TestClientMixin
from grpc import StatusCode
from grpc._channel import _RPCState, _SingleThreadedRendezvous as _Rendezvous

from hermes.client import MessengerClient, MessengerDecoder, NoMessage, SendFailed
from hermes.email import Email, EmailEnvelope


class TestMessengerClient(TestClientMixin, MessengerClient):
    """Testing version of a MessengerClient."""


class MessengerClientTest(TestCase):
    """Unittests for MessengerClient."""

    def setUp(self):
        self.client = TestMessengerClient(sentinel.netloc, secretary_client=sentinel.secretary_client)

    def test_send_default(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template')
        result = self.client.send(message)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_extra_headers(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template',
                        extra_headers={'header': 'value'})
        result = self.client.send(message)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        email.extra_headers['header'] = 'value'
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_context(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        context = {'test': 'test'}

        message = Email('test@example.com', 'subject_template', 'body_template', context=context)
        result = self.client.send(message)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        email.context.update({'test': 'test'})
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_sender(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template', sender='sender@example.com')
        result = self.client.send(message)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.sender = 'sender@example.com'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_body_template_html(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template',
                        body_template_html='html_body_template')
        result = self.client.send(message)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.body_template_html = 'html_body_template'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_template_uuid(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template',
                        body_template_html='html_body_template',
                        subject_template_uuid=UUID(int=1),
                        body_template_uuid=UUID(int=2),
                        body_template_html_uuid=UUID(int=3))
        result = self.client.send(message)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.body_template_html = 'html_body_template'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        email.subject_template_uuid.value = str(UUID(int=1))
        email.body_template_uuid.value = str(UUID(int=2))
        email.body_template_html_uuid.value = str(UUID(int=3))
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_archive(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template')
        result = self.client.send(message, archive=True)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_no_archive(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template')
        result = self.client.send(message, archive=False)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        request = SendEmailRequest(message_data=email, archive=False)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_attachments(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template', attachments=['15', '16'])
        result = self.client.send(message)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        email.attachments.add(value='15')
        email.attachments.add(value='16')
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_failed(self):
        message = Email('test@example.com', 'subject_template', 'body_template', attachments=['15', '16'])
        error = _Rendezvous(_RPCState((), '', '', StatusCode.UNKNOWN, ""), None, None, None)
        self.client.mock.side_effect = error
        with self.assertRaises(SendFailed):
            self.client.send(message)

    def test_get(self):
        reply = GetEmailReply()
        reply.data.envelope.uid.value = 'some uid'
        reply.data.envelope.create_datetime.FromDatetime(datetime(2000, 1, 1, 15, 0, 0))
        reply.data.envelope.message_data.recipient = 'recipient@example.com'
        reply.data.envelope.message_data.body_template = 'body_template'
        reply.data.envelope.message_data.subject_template = 'subject_template'
        self.client.mock.return_value = reply

        result = self.client.get('some uid')

        self.assertIsInstance(result, EmailEnvelope)
        self.assertEqual(result.uid, 'some uid')
        self.assertEqual(result.message.recipient, 'recipient@example.com')

        request = GetEmailRequest()
        request.uid.value = 'some uid'
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/get', timeout=None)])

    def test_get_error(self):
        error = _Rendezvous(_RPCState((), '', '', StatusCode.UNKNOWN, ""), None, None, None)
        self.client.mock.side_effect = error
        with self.assertRaises(grpc.RpcError):
            self.client.get('some uid')

    def test_get_not_found(self):
        error = _Rendezvous(_RPCState((), '', '', StatusCode.NOT_FOUND, ""), None, None, None)
        self.client.mock.side_effect = error
        with self.assertRaises(NoMessage):
            self.client.get('some uid')

    def test_list_empty_request(self):
        reply = ListEmailReply()
        reply.data.envelope.uid.value = 'some uid'
        reply.data.envelope.create_datetime.FromDatetime(datetime(2000, 1, 1, 15, 0, 0))
        reply.data.envelope.message_data.recipient = 'recipient@example.com'
        reply.data.envelope.message_data.body_template = 'body_template'
        reply.data.envelope.message_data.subject_template = 'subject_template'
        self.client.mock.return_value = [reply]

        result = list(self.client.list())

        self.assertEqual(len(result), 1)
        self.assertIsInstance(result[0], EmailEnvelope)
        self.assertEqual(result[0].message.recipient, 'recipient@example.com')
        self.assertEqual(result[0].message.secretary_client, sentinel.secretary_client)

        request = ListEmailRequest()
        self.assertEqual(
            self.client.mock.mock_calls,
            [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/list', timeout=None)]
        )

    def test_list_not_empty_request(self):
        reply = ListEmailReply()
        reply.data.envelope.uid.value = 'some uid'
        reply.data.envelope.create_datetime.FromDatetime(datetime(2000, 1, 1, 15, 0, 0))
        reply.data.envelope.message_data.recipient = 'recipient@example.com'
        reply.data.envelope.message_data.body_template = 'body_template'
        reply.data.envelope.message_data.subject_template = 'subject_template'
        self.client.mock.return_value = [reply]

        result = list(self.client.list(
            created_from=datetime(2000, 1, 1),
            created_to=datetime(2000, 1, 2),
            recipients=['recipient@example.com'],
            body_templates=['body_template', 'body_template_2'],
            types=['techcheck'],
            limit=10,
        ))

        self.assertEqual(len(result), 1)
        self.assertIsInstance(result[0], EmailEnvelope)
        self.assertEqual(result[0].message.recipient, 'recipient@example.com')
        self.assertEqual(result[0].message.secretary_client, sentinel.secretary_client)

        request = ListEmailRequest()
        request.created_from.FromDatetime(datetime(2000, 1, 1))
        request.created_to.FromDatetime(datetime(2000, 1, 2))
        request.recipients[:] = ['recipient@example.com']
        request.body_templates[:] = ['body_template', 'body_template_2']
        request.types[:] = ['techcheck']
        request.limit = 10
        self.assertEqual(
            self.client.mock.mock_calls,
            [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/list', timeout=None)]
        )


class MessengerDecoderTest(TestCase):
    """Unittests for MessengerDecoder."""

    def setUp(self):
        self.decoder = MessengerDecoder()

    def test_decode_uuid(self):
        self.assertEqual(self.decoder.decode(Uuid(value=None)), None)
        self.assertEqual(self.decoder.decode(Uuid(value='')), None)
        self.assertEqual(self.decoder.decode(Uuid(value=str(UUID(int=42)))), UUID(int=42))

==> fred-hermes-0.5.0.tar.gz: fred-hermes-0.5.0/hermes/tests/test_client.py <==
"""Unittests for MessengerClient."""
from datetime import datetime
from typing import Any, Type, cast
from unittest import TestCase
from unittest.mock import call, sentinel
from uuid import UUID

import grpc
from fred_api.messenger.common_types_pb2 import Reference, ReferenceType, Uuid
from fred_api.messenger.email_pb2 import Email as MessengerEmail
from fred_api.messenger.service_email_grpc_pb2 import (GetEmailReply, GetEmailRequest, ListEmailReply, ListEmailRequest,
                                                       SendEmailReply, SendEmailRequest)
from fred_api.messenger.service_sms_grpc_pb2 import (GetSmsReply, GetSmsRequest, ListSmsReply, ListSmsRequest,
                                                     SendSmsReply, SendSmsRequest)
from fred_api.messenger.sms_pb2 import Sms as MessengerSms
from frgal.utils import TestClientMixin
from grpc import StatusCode
from grpc._channel import _RPCState, _SingleThreadedRendezvous as _Rendezvous

from hermes.client import MessengerClient, MessengerDecoder, NoMessage, SendFailed
from hermes.email import Email, EmailEnvelope
from hermes.message import Message, MessageEnvelope
from hermes.sms import Sms, SmsEnvelope


class TestMessengerClient(TestClientMixin, MessengerClient):
    """Testing version of a MessengerClient."""


class MessengerClientTestMixin:
    """Unittests for MessengerClient."""

    get_message_method = None  # type: str
    get_message_request_type = None  # type: Any
    get_message_reply_type = None  # type: Any
    list_messages_method = None  # type: str
    list_messages_request_type = None  # type: Any
    list_messages_reply_type = None  # type: Any
    message_cls = None  # type: Type[Message]
    envelope_cls = None  # type: Type[MessageEnvelope]
    grpc_service_path = None  # type: str

    def setUp(self):
        self.client = TestMessengerClient(sentinel.netloc, secretary_client=sentinel.secretary_client)

    def test_get_message(self):
        reply = self.get_message_reply_type()
        reply.data.envelope.uid.value = 'some uid'
        reply.data.envelope.create_datetime.FromDatetime(datetime(2000, 1, 1, 15, 0, 0))
        reply.data.envelope.message_data.recipient = 'recipient'
        reply.data.envelope.message_data.body_template = 'body_template'
        self.client.mock.return_value = reply

        result = getattr(self.client, self.get_message_method)('some uid')

        cast(TestCase, self).assertIsInstance(result, self.envelope_cls)
        cast(TestCase, self).assertEqual(result.uid, 'some uid')
        cast(TestCase, self).assertEqual(result.message.recipient, 'recipient')

        request = self.get_message_request_type()
        request.uid.value = 'some uid'
        cast(TestCase, self).assertEqual(
            self.client.mock.mock_calls,
            [call(request, method='{}/get'.format(self.grpc_service_path), timeout=None)]
        )

    def test_get_message_error(self):
        error = _Rendezvous(_RPCState((), '', '', StatusCode.UNKNOWN, ""), None, None, None)
        self.client.mock.side_effect = error
        with cast(TestCase, self).assertRaises(grpc.RpcError):
            getattr(self.client, self.get_message_method)('some uid')

    def test_get_message_not_found(self):
        error = _Rendezvous(_RPCState((), '', '', StatusCode.NOT_FOUND, ""), None, None, None)
        self.client.mock.side_effect = error
        with cast(TestCase, self).assertRaises(NoMessage):
            getattr(self.client, self.get_message_method)('some uid')

    def test_list_message_empty_request(self):
        reply = self.list_messages_reply_type()
        reply.data.envelope.uid.value = 'some uid'
        reply.data.envelope.create_datetime.FromDatetime(datetime(2000, 1, 1, 15, 0, 0))
        reply.data.envelope.message_data.recipient = 'recipient'
        reply.data.envelope.message_data.body_template = 'body_template'
        self.client.mock.return_value = [reply]

        result = list(getattr(self.client, self.list_messages_method)())

        cast(TestCase, self).assertEqual(len(result), 1)
        cast(TestCase, self).assertIsInstance(result[0], self.envelope_cls)
        cast(TestCase, self).assertEqual(result[0].message.recipient, 'recipient')  # type: ignore
        cast(TestCase, self).assertEqual(result[0].message.secretary_client, sentinel.secretary_client)

        request = self.list_messages_request_type()
        cast(TestCase, self).assertEqual(
            self.client.mock.mock_calls,
            [call(request, method='{}/list'.format(self.grpc_service_path), timeout=None)]
        )

    def test_list_message_not_empty_request(self):
        reply = self.list_messages_reply_type()
        reply.data.envelope.uid.value = 'some uid'
        reply.data.envelope.create_datetime.FromDatetime(datetime(2000, 1, 1, 15, 0, 0))
        reply.data.envelope.message_data.recipient = 'recipient'
        reply.data.envelope.message_data.body_template = 'body_template'
        self.client.mock.return_value = [reply]

        result = list(getattr(self.client, self.list_messages_method)(
            created_from=datetime(2000, 1, 1),
            created_to=datetime(2000, 1, 2),
            recipients=['recipient'],
            body_templates=['body_template', 'body_template2'],
            types=['techcheck'],
            limit=10,
        ))

        cast(TestCase, self).assertEqual(len(result), 1)
        cast(TestCase, self).assertIsInstance(result[0], self.envelope_cls)
        cast(TestCase, self).assertEqual(result[0].message.recipient, 'recipient')  # type: ignore
        cast(TestCase, self).assertEqual(result[0].message.secretary_client, sentinel.secretary_client)

        request = self.list_messages_request_type()
        request.created_from.FromDatetime(datetime(2000, 1, 1))
        request.created_to.FromDatetime(datetime(2000, 1, 2))
        request.recipients[:] = ['recipient']
        request.body_templates[:] = ['body_template', 'body_template2']
        request.types[:] = ['techcheck']
        request.limit = 10
        cast(TestCase, self).assertEqual(
            self.client.mock.mock_calls,
            [call(request, method='{}/list'.format(self.grpc_service_path), timeout=None)]
        )


class MessengerClientEmailTest(MessengerClientTestMixin, TestCase):
    """Unittests for MessengerClient email methods."""

    get_message_method = 'get_email'
    get_message_request_type = GetEmailRequest
    get_message_reply_type = GetEmailReply
    list_messages_method = 'list_emails'
    list_messages_request_type = ListEmailRequest
    list_messages_reply_type = ListEmailReply
    message_cls = Email
    envelope_cls = EmailEnvelope
    grpc_service_path = '/Fred.Messenger.Api.Email.EmailMessenger'

    def test_send_default(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template')
        result = self.client.send(message)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_type(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template', type='techcheck')
        result = self.client.send(message)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        email.type = 'techcheck'
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_extra_headers(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template',
                        extra_headers={'header': 'value'})
        result = self.client.send(message)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        email.extra_headers['header'] = 'value'
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_context(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        context = {'test': 'test'}

        message = Email('test@example.com', 'subject_template', 'body_template', context=context)
        result = self.client.send(message)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        email.context.update({'test': 'test'})
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_sender(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template', sender='sender@example.com')
        result = self.client.send(message)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.sender = 'sender@example.com'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_body_template_html(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template',
                        body_template_html='html_body_template')
        result = self.client.send(message)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.body_template_html = 'html_body_template'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_template_uuid(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template',
                        body_template_html='html_body_template',
                        subject_template_uuid=UUID(int=1),
                        body_template_uuid=UUID(int=2),
                        body_template_html_uuid=UUID(int=3))
        result = self.client.send(message)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.body_template_html = 'html_body_template'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        email.subject_template_uuid.value = str(UUID(int=1))
        email.body_template_uuid.value = str(UUID(int=2))
        email.body_template_html_uuid.value = str(UUID(int=3))
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_archive(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template')
        result = self.client.send(message, archive=True)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_no_archive(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template')
        result = self.client.send(message, archive=False)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        request = SendEmailRequest(message_data=email, archive=False)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_attachments(self):
        reply = SendEmailReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Email('test@example.com', 'subject_template', 'body_template', attachments=['15', '16'])
        result = self.client.send(message)

        self.assertEqual(result, '42')

        email = MessengerEmail()
        email.recipient = 'test@example.com'
        email.subject_template = 'subject_template'
        email.body_template = 'body_template'
        email.attachments.add(value='15')
        email.attachments.add(value='16')
        request = SendEmailRequest(message_data=email, archive=True)
        self.assertEqual(self.client.mock.mock_calls,
                         [call(request, method='/Fred.Messenger.Api.Email.EmailMessenger/send', timeout=None)])

    def test_send_failed(self):
        message = Email('test@example.com', 'subject_template', 'body_template', attachments=['15', '16'])
        error = _Rendezvous(_RPCState((), '', '', StatusCode.UNKNOWN, ""), None, None, None)
        self.client.mock.side_effect = error
        with self.assertRaises(SendFailed):
            self.client.send(message)


class MessengerClientSmsTest(MessengerClientTestMixin, TestCase):
    """Unittests for MessengerClient sms methods."""

    get_message_method = 'get_sms'
    get_message_request_type = GetSmsRequest
    get_message_reply_type = GetSmsReply
    list_messages_method = 'list_sms'
    list_messages_request_type = ListSmsRequest
    list_messages_reply_type = ListSmsReply
    message_cls = Sms
    envelope_cls = SmsEnvelope
    grpc_service_path = '/Fred.Messenger.Api.Sms.SmsMessenger'

    def test_send_default(self):
        reply = SendSmsReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Sms('+420.999888777', 'body_template')
        result = self.client.send_sms(message)

        self.assertEqual(result, '42')

        sms = MessengerSms()
        sms.recipient = '+420.999888777'
        sms.body_template = 'body_template'
        request = SendSmsRequest(message_data=sms, archive=True)
        self.assertEqual(
            self.client.mock.mock_calls,
            [call(request, method='{}/send'.format(self.grpc_service_path), timeout=None)]
        )

    def test_send_full(self):
        reply = SendSmsReply()
        reply.data.uid.value = '42'
        self.client.mock.return_value = reply

        message = Sms('+420.999888777', 'body_template', type='techcheck', body_template_uuid=UUID(int=1))
        result = self.client.send_sms(message, archive=False)

        self.assertEqual(result, '42')

        sms = MessengerSms()
        sms.recipient = '+420.999888777'
        sms.body_template = 'body_template'
        sms.type = 'techcheck'
        sms.body_template_uuid.value = str(UUID(int=1))
        request = SendSmsRequest(message_data=sms, archive=False)
        self.assertEqual(
            self.client.mock.mock_calls,
            [call(request, method='{}/send'.format(self.grpc_service_path), timeout=None)]
        )

    def test_send_failed(self):
        message = Sms('+420.999888777', 'body_template')
        error = _Rendezvous(_RPCState((), '', '', StatusCode.UNKNOWN, ""), None, None, None)
        self.client.mock.side_effect = error
        with self.assertRaises(SendFailed):
            self.client.send_sms(message)


class MessengerClientDeprecatedEmailTest(MessengerClientTestMixin, TestCase):
    """Unittests for MessengerClient deprecated email methods."""

    get_message_method = 'get'
    get_message_request_type = GetEmailRequest
    get_message_reply_type = GetEmailReply
    list_messages_method = 'list'
    list_messages_request_type = ListEmailRequest
    list_messages_reply_type = ListEmailReply
    message_cls = Email
    envelope_cls = EmailEnvelope
    grpc_service_path = '/Fred.Messenger.Api.Email.EmailMessenger'


class MessengerDecoderTest(TestCase):
    """Unittests for MessengerDecoder."""

    def setUp(self):
        self.decoder = MessengerDecoder()

    def test_decode_reference(self):
        self.assertEqual(
            self.decoder.decode(Reference(value=str(UUID(int=1)), type=ReferenceType.Value('domain'))),
            {'value': UUID(int=1), 'type': 'domain'}
        )
        self.assertEqual(
            self.decoder.decode(Reference(value='REG-NIC', type=ReferenceType.Value('registrar'))),
            {'value': 'REG-NIC', 'type': 'registrar'}
        )

    def test_decode_uuid(self):
        self.assertEqual(self.decoder.decode(Uuid(value=None)), None)
        self.assertEqual(self.decoder.decode(Uuid(value='')), None)
        self.assertEqual(self.decoder.decode(Uuid(value=str(UUID(int=42)))), UUID(int=42))
